@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -85,13 +85,10 @@ static GSList *filesel_select_file_full(const gchar *title, const gchar *file,
 	if ((cwd = g_hash_table_lookup(path_table, title)) != NULL)
 		change_dir(cwd);
 	else
-		change_dir(startup_dir);
+		change_dir(get_startup_dir());
 
 	dialog = filesel_create(title, action);
 
-	change_dir(prev_dir);
-	g_free(prev_dir);
-
 	manage_window_set_transient(GTK_WINDOW(dialog));
 
 	if (file)
@@ -108,6 +105,9 @@ static GSList *filesel_select_file_full(const gchar *title, const gchar *file,
 
 	gtk_widget_show(dialog);
 
+	change_dir(prev_dir);
+	g_free(prev_dir);
+
 	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
 		list = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dialog));
 		if (list) {
@@ -151,6 +151,23 @@ gchar *filesel_save_as(const gchar *file)
 	return filename;
 }
 
+gchar *filesel_select_dir(const gchar *dir)
+{
+	GSList *list;
+	gchar *selected = NULL;
+
+	list = filesel_select_file_full(_("Select directory"), dir,
+					GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+					FALSE);
+	if (list) {
+		selected = (gchar *)list->data;
+		slist_free_strings(list->next);
+	}
+	g_slist_free(list);
+
+	return selected;
+}
+
 static GtkWidget *filesel_create(const gchar *title,
 				 GtkFileChooserAction action)
 {
@@ -160,17 +177,20 @@ static GtkWidget *filesel_create(const gchar *title,
 		dialog = gtk_file_chooser_dialog_new
 			(title, NULL, action,
 			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			 action == GTK_FILE_CHOOSER_ACTION_SAVE ? GTK_STOCK_SAVE
-			 : GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+			 (action == GTK_FILE_CHOOSER_ACTION_SAVE ||
+			  action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)
+			 ? GTK_STOCK_SAVE : GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 			 NULL);
 	else
 		dialog = gtk_file_chooser_dialog_new
 			(title, NULL, action,
-			 action == GTK_FILE_CHOOSER_ACTION_SAVE ? GTK_STOCK_SAVE
-			 : GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+			 (action == GTK_FILE_CHOOSER_ACTION_SAVE ||
+			  action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)
+			 ? GTK_STOCK_SAVE : GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			 NULL);
-	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
+	gtk_window_set_position(GTK_WINDOW(dialog),
+				GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
 	gtk_window_set_wmclass
 		(GTK_WINDOW(dialog), "file_selection", "Sylpheed");
