@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,12 +26,16 @@
 #include <gtk/gtkwidget.h>
 #include <gtk/gtktreestore.h>
 #include <gtk/gtktreeview.h>
+#include <gtk/gtktreeselection.h>
+#include <gtk/gtkcellrendererpixbuf.h>
+#include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkstock.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -50,13 +54,15 @@
 #include "folderview.h"
 #include "inputdialog.h"
 #include "folder.h"
+#include "prefs_common.h"
 
 enum {
 	FOLDERSEL_FOLDERNAME,
 	FOLDERSEL_FOLDERITEM,
 	FOLDERSEL_PIXBUF,
 	FOLDERSEL_PIXBUF_OPEN,
-	FOLDERSEL_EXPANDER,
+	FOLDERSEL_FOREGROUND,
+	FOLDERSEL_BOLD,
 	N_FOLDERSEL_COLUMNS
 };
 
@@ -76,14 +82,17 @@ static GdkPixbuf *foldernoselect_pixbuf;
 static GtkWidget *window;
 static GtkWidget *treeview;
 static GtkWidget *entry;
+static GtkWidget *confirm_area;
 static GtkWidget *ok_button;
 static GtkWidget *cancel_button;
 static GtkWidget *new_button;
 
+static GtkTreeStore *tree_store;
+
 static FolderItem *folder_item;
 static FolderItem *selected_item;
 
-static GtkTreeStore *tree_store;
+FolderSelectionType sel_type;
 
 static gboolean cancelled;
 static gboolean finished;
@@ -96,8 +105,7 @@ static void foldersel_append_item	(GtkTreeStore	*store,
 					 GtkTreeIter	*iter,
 					 GtkTreeIter	*parent);
 
-static void foldersel_set_tree		(Folder			*cur_folder,
-					 FolderSelectionType	 type);
+static void foldersel_set_tree		(Folder		*cur_folder);
 
 static gboolean foldersel_selected	(GtkTreeSelection *selection,
 					 GtkTreeModel	  *model,
@@ -141,13 +149,14 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 				 const gchar *default_folder)
 {
 	selected_item = NULL;
+	sel_type = type;
 
 	if (!window) {
 		foldersel_create();
 		foldersel_init();
 	}
 
-	foldersel_set_tree(cur_folder, type);
+	foldersel_set_tree(cur_folder);
 
 	/* select current */
 	if (folder_item) {
@@ -180,11 +189,13 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 	} else
 		gtk_tree_view_scroll_to_point(GTK_TREE_VIEW(treeview), 0, 0);
 
+	gtkut_box_set_reverse_order(GTK_BOX(confirm_area),
+				    !prefs_common.comply_gnome_hig);
 	gtk_widget_grab_focus(ok_button);
 	gtk_widget_grab_focus(treeview);
 
-	gtk_widget_show(window);
 	manage_window_set_transient(GTK_WINDOW(window));
+	gtk_widget_show(window);
 
 	cancelled = finished = FALSE;
 
@@ -195,19 +206,22 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 	gtk_entry_set_text(GTK_ENTRY(entry), "");
 	gtk_tree_store_clear(tree_store);
 
-	if (!cancelled &&
-	    selected_item && selected_item->path && !selected_item->no_select) {
+	if (cancelled || !selected_item)
+		return NULL;
+
+	if (type == FOLDER_SEL_ALL || type == FOLDER_SEL_MOVE_FOLDER ||
+	    (selected_item->path && !selected_item->no_select)) {
 		folder_item = selected_item;
 		return folder_item;
-	} else
-		return NULL;
+	}
+
+	return NULL;
 }
 
 static void foldersel_create(void)
 {
 	GtkWidget *vbox;
 	GtkWidget *scrolledwin;
-	GtkWidget *confirm_area;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 	GtkTreeSelection *selection;
@@ -215,7 +229,8 @@ static void foldersel_create(void)
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(window), _("Select folder"));
 	gtk_container_set_border_width(GTK_CONTAINER(window), 4);
-	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
+	gtk_window_set_position(GTK_WINDOW(window),
+				GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, FALSE);
 	gtk_window_set_wmclass
@@ -243,7 +258,8 @@ static void foldersel_create(void)
 					G_TYPE_POINTER,
 					GDK_TYPE_PIXBUF,
 					GDK_TYPE_PIXBUF,
-					G_TYPE_BOOLEAN);
+					GDK_TYPE_COLOR,
+					G_TYPE_INT);
 	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(tree_store),
 					FOLDERSEL_FOLDERNAME,
 					foldersel_folder_name_compare,
@@ -272,7 +288,6 @@ static void foldersel_create(void)
 	gtk_tree_view_column_pack_start(column, renderer, FALSE);
 	gtk_tree_view_column_set_attributes
 		(column, renderer,
-		 "is-expander", FOLDERSEL_EXPANDER,
 		 "pixbuf", FOLDERSEL_PIXBUF,
 		 "pixbuf-expander-open", FOLDERSEL_PIXBUF_OPEN,
 		 "pixbuf-expander-closed", FOLDERSEL_PIXBUF,
@@ -281,9 +296,12 @@ static void foldersel_create(void)
 	/* create text renderer */
 	renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_column_pack_start(column, renderer, TRUE);
-	gtk_tree_view_column_set_attributes(column, renderer,
-					    "text", FOLDERSEL_FOLDERNAME,
-					    NULL);
+	gtk_tree_view_column_set_attributes
+		(column, renderer,
+		 "text", FOLDERSEL_FOLDERNAME,
+		 "foreground-gdk", FOLDERSEL_FOREGROUND,
+		 "weight", FOLDERSEL_BOLD,
+		 NULL);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 
 	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
@@ -325,8 +343,13 @@ static void foldersel_init(void)
 static void foldersel_append_item(GtkTreeStore *store, FolderItem *item,
 				  GtkTreeIter *iter, GtkTreeIter *parent)
 {
-	const gchar *name;
+	gchar *name;
 	GdkPixbuf *pixbuf, *pixbuf_open;
+	gboolean use_color;
+	PangoWeight weight = PANGO_WEIGHT_NORMAL;
+	GdkColor *foreground = NULL;
+	static GdkColor color_noselect = {0, COLOR_DIM, COLOR_DIM, COLOR_DIM};
+	static GdkColor color_new = {0, (guint16)55000, 15000, 15000};
 
 	name = item->name;
 
@@ -370,21 +393,46 @@ static void foldersel_append_item(GtkTreeStore *store, FolderItem *item,
 		}
 	}
 
-	gtk_tree_store_append(store, iter, parent);
+	if (item->stype == F_QUEUE && item->total > 0) {
+		name = g_strdup_printf("%s (%d)", name, item->total);
+	} else if (item->unread > 0) {
+		name = g_strdup_printf("%s (%d)", name, item->unread);
+	} else
+		name = g_strdup(name);
 
-	pixbuf = item->no_select ? foldernoselect_pixbuf : folder_pixbuf,
+	pixbuf = item->no_select ? foldernoselect_pixbuf : folder_pixbuf;
 	pixbuf_open =
-		item->no_select ? foldernoselect_pixbuf : folderopen_pixbuf,
+		item->no_select ? foldernoselect_pixbuf : folderopen_pixbuf;
+
+	if (item->stype == F_OUTBOX || item->stype == F_DRAFT ||
+	    item->stype == F_TRASH) {
+		use_color = FALSE;
+	} else if (item->stype == F_QUEUE) {
+		use_color = (item->total > 0);
+		if (item->total > 0)
+			weight = PANGO_WEIGHT_BOLD;
+	} else {
+		use_color = (item->new > 0);
+		if (item->unread > 0)
+			weight = PANGO_WEIGHT_BOLD;
+	}
+
+	if (item->no_select)
+		foreground = &color_noselect;
+	else if (use_color)
+		foreground = &color_new;
 
 	/* insert this node */
+	gtk_tree_store_append(store, iter, parent);
 	gtk_tree_store_set(store, iter,
 			   FOLDERSEL_FOLDERNAME, name,
 			   FOLDERSEL_FOLDERITEM, item,
 			   FOLDERSEL_PIXBUF, pixbuf,
 			   FOLDERSEL_PIXBUF_OPEN, pixbuf_open,
-			   FOLDERSEL_EXPANDER,
-				item->node->children ? TRUE : FALSE,
+			   FOLDERSEL_FOREGROUND, foreground,
+			   FOLDERSEL_BOLD, weight,
 			   -1);
+	g_free(name);
 }
 
 static void foldersel_insert_gnode_in_store(GtkTreeStore *store, GNode *node,
@@ -406,7 +454,7 @@ static void foldersel_insert_gnode_in_store(GtkTreeStore *store, GNode *node,
 		foldersel_insert_gnode_in_store(store, iter, &child);
 }
 
-static void foldersel_set_tree(Folder *cur_folder, FolderSelectionType type)
+static void foldersel_set_tree(Folder *cur_folder)
 {
 	Folder *folder;
 	GList *list;
@@ -415,10 +463,12 @@ static void foldersel_set_tree(Folder *cur_folder, FolderSelectionType type)
 		folder = FOLDER(list->data);
 		g_return_if_fail(folder != NULL);
 
-		if (type != FOLDER_SEL_ALL) {
+		if (sel_type != FOLDER_SEL_ALL) {
 			if (FOLDER_TYPE(folder) == F_NEWS)
 				continue;
 		}
+		if (sel_type == FOLDER_SEL_MOVE_FOLDER && folder != cur_folder)
+			continue;
 
 		foldersel_insert_gnode_in_store(tree_store, folder->node, NULL);
 	}
@@ -447,7 +497,9 @@ static gboolean foldersel_selected(GtkTreeSelection *selection,
 			   FOLDERSEL_FOLDERITEM, &item, -1);
 
 	selected_item = item;
-	if (selected_item && selected_item->path && !selected_item->no_select) {
+	if (selected_item &&
+	    (sel_type == FOLDER_SEL_ALL || sel_type == FOLDER_SEL_MOVE_FOLDER ||
+	     (selected_item->path && !selected_item->no_select))) {
 		gchar *id;
 		id = folder_item_get_identifier(selected_item);
 		gtk_entry_set_text(GTK_ENTRY(entry), id);
@@ -521,10 +573,6 @@ static void foldersel_new_folder(GtkButton *button, gpointer data)
 		return;
 	}
 
-	/* parent can expand */
-	gtk_tree_store_set(store, &selected, FOLDERSEL_EXPANDER, TRUE,
-			   -1);
-
 	/* add new child */
 	foldersel_append_item(store, new_item, &new_child, &selected);
 	selected_p = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &selected);
@@ -538,7 +586,7 @@ static void foldersel_new_folder(GtkButton *button, gpointer data)
 	gtk_tree_path_free(new_child_p);
 	gtk_tree_path_free(selected_p);
 
-	folderview_append_item(folderview_get(), NULL, new_item);
+	folderview_append_item(folderview_get(), NULL, new_item, TRUE);
 	folder_write_list();
 }
 
