@@ -349,7 +349,7 @@ void prefs_actions_read_config(void)
 	debug_print("Reading actions configurations...\n");
 
 	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, ACTIONS_RC, NULL);
-	if ((fp = fopen(rcpath, "rb")) == NULL) {
+	if ((fp = g_fopen(rcpath, "rb")) == NULL) {
 		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
 		g_free(rcpath);
 		return;
@@ -655,6 +655,7 @@ static void prefs_actions_cancel(GtkWidget *w, gpointer data)
 {
 	prefs_actions_read_config();
 	gtk_widget_hide(actions.window);
+	main_window_popup(main_window_get());
 	inc_unlock();
 }
 
@@ -667,6 +668,7 @@ static void prefs_actions_ok(GtkWidget *widget, gpointer data)
 	ifactory = gtk_item_factory_from_widget(mainwin->menubar);
 	action_update_mainwin_menu(ifactory, mainwin);
 	gtk_widget_hide(actions.window);
+	main_window_popup(main_window_get());
 	inc_unlock();
 }
 
