@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,37 +31,15 @@
 typedef struct _SummaryView		SummaryView;
 typedef struct _SummaryColumnState	SummaryColumnState;
 
+#include "enums.h"
 #include "mainwindow.h"
 #include "folderview.h"
 #include "headerview.h"
 #include "messageview.h"
 #include "compose.h"
-#include "prefs_filter.h"
+#include "filter.h"
 #include "folder.h"
-
-typedef enum
-{
-	S_COL_MARK,
-	S_COL_UNREAD,
-	S_COL_MIME,
-	S_COL_SUBJECT,
-	S_COL_FROM,
-	S_COL_DATE,
-	S_COL_SIZE,
-	S_COL_NUMBER,
-
-	S_COL_MSG_INFO,
-
-	S_COL_LABEL,
-	S_COL_TO,
-
-	S_COL_FOREGROUND,
-	S_COL_BOLD,
-
-	N_SUMMARY_COLS
-} SummaryColumnType;
-
-#define N_SUMMARY_VISIBLE_COLS	S_COL_MSG_INFO
+#include "procmsg.h"
 
 typedef enum
 {
@@ -80,6 +58,14 @@ struct _SummaryColumnState
 struct _SummaryView
 {
 	GtkWidget *vbox;
+
+	GtkWidget *search_hbox;
+	GtkWidget *filter_optmenu;
+	GtkWidget *filter_menu;
+	GtkWidget *search_label;
+	GtkWidget *search_entry;
+	GtkWidget *search_clear_btn;
+
 	GtkWidget *scrolledwin;
 	GtkWidget *treeview;
 
@@ -87,6 +73,7 @@ struct _SummaryView
 	GtkTreeSelection *selection;
 	GtkTreeViewColumn *columns[N_SUMMARY_VISIBLE_COLS];
 
+	GtkWidget *hseparator;
 	GtkWidget *hbox;
 	GtkWidget *hbox_l;
 	GtkWidget *statlabel_folder;
@@ -127,10 +114,10 @@ struct _SummaryView
 	FolderItem *folder_item;
 
 	/* current message status */
-	off_t total_size;
-	gint  deleted;
-	gint  moved;
-	gint  copied;
+	gint64 total_size;
+	gint deleted;
+	gint moved;
+	gint copied;
 
 /*
 private:
@@ -138,12 +125,32 @@ private:
 	/* table for looking up message-id */
 	GHashTable *msgid_table;
 
+	/* all message list */
+	GSList *all_mlist;
+	/* filtered message list */
+	GSList *flt_mlist;
+
+	gint64 total_flt_msg_size;
+	gint flt_msg_total;
+	gint flt_deleted;
+	gint flt_moved;
+	gint flt_copied;
+	gint flt_new;
+	gint flt_unread;
+
+	gboolean on_filter;
+
 	/* list for moving/deleting messages */
-	GSList *mlist;
+	GSList *tmp_mlist;
+	FolderItem *to_folder;
 	/* table for updating folder tree */
 	GHashTable *folder_table;
 	/* counter for filtering */
 	gint filtered;
+	gint flt_count;
+	gint flt_total;
+
+	gboolean on_button_press;
 
 	/* for DnD */
 	gboolean can_toggle_selection;
@@ -170,6 +177,8 @@ gboolean summary_is_locked	  (SummaryView		*summaryview);
 
 SummarySelection summary_get_selection_type	(SummaryView	*summaryview);
 GSList *summary_get_selected_msg_list		(SummaryView	*summaryview);
+GSList *summary_get_changed_msg_list		(SummaryView	*summaryview);
+GSList *summary_get_msg_list			(SummaryView	*summaryview);
 
 void summary_select_prev_unread	  (SummaryView		*summaryview);
 void summary_select_next_unread	  (SummaryView		*summaryview);
@@ -181,6 +190,8 @@ void summary_select_prev_labeled  (SummaryView		*summaryview);
 void summary_select_next_labeled  (SummaryView		*summaryview);
 void summary_select_by_msgnum	  (SummaryView		*summaryview,
 				   guint		 msgnum);
+gboolean summary_select_by_msginfo(SummaryView		*summaryview,
+				   MsgInfo		*msginfo);
 void summary_select_row		  (SummaryView		*summaryview,
 				   GtkTreeIter		*iter,
 				   gboolean		 display_msg,
@@ -195,7 +206,7 @@ void summary_collapse_threads	  (SummaryView		*summaryview);
 void summary_filter		  (SummaryView		*summaryview,
 				   gboolean		 selected_only);
 void summary_filter_open	  (SummaryView		*summaryview,
-				   PrefsFilterType	 type);
+				   FilterCreateType	 type);
 
 void summary_filter_junk	  (SummaryView		*summaryview,
 				   gboolean		 selected_only);
@@ -244,6 +255,7 @@ void summary_mark		  (SummaryView		*summaryview);
 void summary_unmark		  (SummaryView		*summaryview);
 void summary_mark_as_unread	  (SummaryView		*summaryview);
 void summary_mark_as_read	  (SummaryView		*summaryview);
+void summary_mark_thread_as_read  (SummaryView		*summaryview);
 void summary_mark_all_read	  (SummaryView		*summaryview);
 void summary_add_address	  (SummaryView		*summaryview);
 void summary_select_all		  (SummaryView		*summaryview);
@@ -260,4 +272,8 @@ void summary_set_colorlabel	  (SummaryView		*summaryview,
 void summary_set_column_order	  (SummaryView		*summaryview);
 void summary_get_column_order	  (SummaryView		*summaryview);
 
+void summary_qsearch_reset	  (SummaryView		*summaryview);
+void summary_qsearch_clear_entry  (SummaryView		*summaryview);
+void summary_qsearch		  (SummaryView		*summaryview);
+
 #endif /* __SUMMARY_H__ */
